;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10 -*-
;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          mop.lisp
;;;; Purpose:       Imports standard MOP symbols into KMRCL
;;;; Programmer:    Kevin M. Rosenberg
;;;; Date Started:  Apr 2003
;;;;
;;;; $Id: mop.lisp,v 1.3 2003/04/29 04:56:58 kevin Exp $
;;;;
;;;; This file, part of KMRCL, is Copyright (c) 2002 by Kevin M. Rosenberg
;;;;
;;;; KMRCL users are granted the rights to distribute and use this software
;;;; as governed by the terms of the Lisp Lesser GNU Public License
;;;; (http://opensource.franz.com/preamble.html), also known as the LLGPL.
;;;; *************************************************************************

;;; This file imports MOP symbols into KMR-MOP packages and then
;;; re-exports them to hide differences in MOP implementations.

(in-package #:cl-user)

#+sbcl
(eval-when (:compile-toplevel :load-toplevel :execute)
  (if (find-package 'sb-mop)
      (pushnew :kmr-sbcl-mop cl:*features*)
      (pushnew :kmr-sbcl-pcl cl:*features*)))

#+cmu
(eval-when (:compile-toplevel :load-toplevel :execute)
  (if (find-package 'mop)
      (pushnew :kmr-cmucl-mop cl:*features*)
      (pushnew :kmr-cmucl-pcl cl:*features*)))

(defpackage #:kmr-mop
  (:use
   #:cl
   #:kmrcl
   #+kmr-sbcl-mop #:sb-mop
   #+kmr-cmucl-mop #:mop
   #+allegro #:mop
   #+lispworks #:clos
   #+scl #:clos)
  (:export
   #:class-of #:class-name #:class-slots #:find-class
   #:standard-class
   #:slot-definition-name #:finalize-inheritance
   #:standard-direct-slot-definition
   #:standard-effective-slot-definition #:validate-superclass
   #:direct-slot-definition-class #:compute-effective-slot-definition
   #:compute-effective-slot-definition-initargs
   #:slot-value-using-class
   #:class-prototype #:generic-function-method-class #:intern-eql-specializer
   #:make-method-lambda #:generic-function-lambda-list
   #:compute-slots)   
   )

(in-package #:kmr-mop)

(shadowing-import
 #+allegro
 '(excl::compute-effective-slot-definition-initargs)
 #+lispworks
 '(clos::compute-effective-slot-definition-initargs)
 #+kmr-sbcl-mop 
 '(sb-pcl::compute-effective-slot-definition-initargs)
 #+kmr-sbcl-pcl
 '(sb-pcl:class-of sb-pcl:class-name sb-pcl:class-slots sb-pcl:find-class
   sb-pcl::standard-class
   sb-pcl:slot-definition-name sb-pcl::finalize-inheritance
   sb-pcl::standard-direct-slot-definition
   sb-pcl::standard-effective-slot-definition sb-pcl::validate-superclass
   sb-pcl::direct-slot-definition-class sb-pcl::compute-effective-slot-definition
   sb-pcl::compute-effective-slot-definition-initargs
   sb-pcl::slot-value-using-class
   sb-pcl:class-prototype sb-pcl:generic-function-method-class sb-pcl:intern-eql-specializer
   sb-pcl:make-method-lambda sb-pcl:generic-function-lambda-list
   sb-pcl::compute-slots)
 #+kmr-cmucl-mop 
 '(pcl::compute-effective-slot-definition-initargs)
 #+kmr-cmucl-pcl
 '(pcl:class-of  pcl:class-name pcl:class-slots pcl:find-class pcl::standard-class
   pcl::slot-definition-name pcl:finalize-inheritance
   pcl::standard-direct-slot-definition pcl::standard-effective-slot-definition
   pcl::validate-superclass pcl:direct-slot-definition-class
   pcl:compute-effective-slot-definition
   pcl::compute-effective-slot-definition-initargs
   pcl::slot-value-using-class
   pcl:class-prototype pcl:generic-function-method-class pcl:intern-eql-specializer
   pcl:make-method-lambda pcl:generic-function-lambda-list
   pcl::compute-slots)
 #+scl
 '(clos::compute-effective-slot-definition-initargs
   clos::class-prototype
   ;; note: make-method-lambda is not fbound
   )
 '#:kmr-mop)

#+sbcl
(eval-when (:compile-toplevel :load-toplevel :execute)
  (if (find-package 'sb-mop)
      (setq cl:*features* (delete :kmr-sbcl-mop cl:*features*))
      (setq cl:*features* (delete :kmr-sbcl-pcl cl:*features*))))

#+cmu
(eval-when (:compile-toplevel :load-toplevel :execute)
  (if (find-package 'mop)
      (setq cl:*features* (delete :kmr-cmucl-mop cl:*features*))
      (setq cl:*features* (delete :kmr-cmucl-pcl cl:*features*))))

(eval-when (:compile-toplevel :load-toplevel :execute)
  (when (>= (length (generic-function-lambda-list
		     (ensure-generic-function
		      'compute-effective-slot-definition)))
	    3)
    (pushnew :kmr-normal-cesd cl:*features*)))

(eval-when (:compile-toplevel :load-toplevel :execute)
  (when (>= (length (generic-function-lambda-list
		    (ensure-generic-function
		     'direct-slot-definition-class)))
	    3)
    (pushnew :kmr-normal-dsdc cl:*features*)))

